function dat = DRMdat( ifo )

if ifo.optics.findNamedOptic('BS') == 0
    % FP no BS
    dat = modeDat( ifo, 'ITM-HR-o');
    dat.RTL = ifo.roundtripLoss( 'ITM-HR-o' );
else
    % PRM or DRM with BS
    dat.PRG = ifo.power('BS-HRx-o');
    dat.Dark = DarkData( ifo );
    dat.Xarm = oneArmData(ifo, 'ITMX');
    dat.Yarm = oneArmData(ifo, 'ITMY');
end

return

function dat = DarkData( ifo )
dat = modeDat( ifo, 'BS-ARy-o' );
% add field data
[EB,xB] = ifo.getField('BS-ARy-o');
ind = find( abs(xB) < 0.37/2 );
if mod(ind,2) ~= 0
    ind = ind(1):ind(end)+1;
end
dat.x = xB(ind);
dat.E = EB(ind,ind);
return

function dat = modeDat( ifo, FldID )
dat.power = ifo.power(FldID);
dat.mode = ifo.mainHGs(FldID);
dat.HGmodes = ifo.HGPower(FldID,-10,-1);
dat.HG7Coef = ifo.HGCoef(FldID,-7,1);
dat.LG7Coef = ifo.LGCoef(FldID,-7,1);
return

function dat = oneArmData( ifo, ITM )

dat.HR = modeDat( ifo, strcat(ITM, '-HR-o') );
dat.AR = modeDat( ifo, strcat(ITM, '-AR-o') );
dat.RTL = ifo.roundtripLoss(strcat(ITM, '-HR-o'));

return
