function [freq, PSD_1D] = LMA_PSD( map, x, y, diam, PSDrxy )
% function [freq, PSD_1D] = LMA_PSD( surface, xl, yl, diam, PSDrxy )
% calculate 1D PSD
%     surface : height of the surface in m
%     xl, yl  : grid points of data
%     diam    : mirror diameter
%     PSDxyr  : 0 : PSD(r) integrated over phi
%               1 : PSD(x) integrated over y
%               2 : PSD(y) integrated over x
%
% originaly PLOT_PSD by Jerome Degallaix <j.degallaix@lma.in2p3.fr>
%
%Plot_PSD Plot the 1D PSD of a surface
% Function used to plot (and return) the 1D PSD derived from an object
% Interface
% Adapted from a set of functions made by M. Galimberti in the years
% 2010-2011 at LMA.

% -------------------------------------------------------------------------------

[freq, ASD_2D] = LMA_ASD( map, x, y, diam );

PSD_2D = abs(ASD_2D).^2 / diam^2;
df = max(diff(freq));

if exist('PSDrxy', 'var') && (PSDrxy == 1 || PSDrxy == 2) % integrate over y or x
    PSD_1D_2s = sum(PSD_2D,PSDrxy) * df;
    PSD_1D = PSD_1D_2s(1:end/2+1) + PSD_1D_2s(end:-1:end/2);
    
else % Pass in 1D by integrating along a constant radius
    rad_au = size(PSD_2D,1)/2;
    [rad_auX, rad_auY] = meshgrid(-rad_au: rad_au-1);
    rad_au2D =sqrt(rad_auX.^2 + rad_auY.^2);
    
    PSD_1D = zeros(1,rad_au);
    PSD_2D = fftshift(PSD_2D); % Bring back the low frequency in the middle of the map
    
    for kk=1:rad_au
        ind_area = rad_au2D >= kk - 1  & rad_au2D < kk;
        PSD_1D(kk) = sum(PSD_2D(ind_area));
    end
    PSD_1D = PSD_1D * df;
    freq(1) = [];
    
end

end

