function mirgrid=R_FakeMirror1D(DiagonalPSDLaw1D, deltax,N,shape,bad)
% This is 1D version of Bondu's function
%    DiagonalPSDLaw1D is a  1D PSD
%    DiagonalPSDLaw1D = 1/(f^2+100) generates mirgrid whose 1D PSD is 1/(f^2+100)
%
% This is exact copy of Bondu's function but with all dependant functions
% added to file
% Large Virtual Mirror Maps : bondu_090409
% mirgrid=FakeMirror2D(deltax,N);
% input: deltax: grid step size
%        N: number of points horizontally and vertically - even !!
%        shape: if 'disc' then out-of-radius points are set to value bad
%        bad: value for out of radius points
%        DiagonalPSDLaw2D : function handle of 1d PSD
% output: a square grid, a disc if shape=='disc'

    if (mod(N,2)==1)
       mirgrid = 0;
       return
    end

    deltaf = 1/(N*deltax);

    % first define in frequency domain on a half plane

    % two squares as bricks to define a half space
    DiagonalPSDLaw2D = @(f) DiagonalPSDLaw1D(f) ./ f / pi;
    matU = SquareDiagonalLSD(DiagonalPSDLaw2D,deltaf,N/2);
    matV = SquareDiagonalLSD(DiagonalPSDLaw2D,deltaf,N/2);

    % first quadrant : simplest one
    matA = matU;

    % fourth quadrant
    matD = conj(matU);
    matD = fliplr(flipud(matD));
    matD(2:end,:) = matD(1:end-1,:);
    matD(:,2:end) = matD(:,1:end-1);
    matD(:,1) = 0; % Nyquist frequency
    matD(1,:) = 0; % Nyquist frequency

    % second quadrant: remove column with DC, 
    %                add line with zero at Nyquist frequency
    matB = matV;
    matB = fliplr(matV);
    matB(:,2:end) = matB(:,1:end-1);
    matB(:,1) = 0;
    matB(1,2:end) = conj(fliplr(matA(1,2:end)));

    % third quadrant:
    matC = flipud(conj(matV));
    matC(2:end,:) = matC(1:end-1,:);
    matC(1,:) = 0;
    matC(2:end,1) = conj(flipud(matA(2:end,1)));

    Freq_mirgrid = [matA matB ; matC matD];

    %mirgrid = fftshift(Freq_mirgrid);

    mirgrid = ifft2(Freq_mirgrid,'symmetric');
    calib = 1/(sqrt(2)*deltax/N);
    mirgrid = real(mirgrid*calib);

    if exist('shape','var') && strcmpi(shape, 'disc')
        middle = round(N/2);
        [indx,indy] = meshgrid(1:N,1:N);
        radius = sqrt((indx-middle).^2+(indy-middle).^2);
        mirgrid(radius>N/2) = bad;
    end
end

function tab=SquareDiagonalLSD(DiagonalPSDLaw2D,deltaf,N)
% generates a square of random numbers
% tab(1,1) = mean = 0
% tab(u,v) is an imaginary number
% |tab(u,v)|^2 is PSD(f) with f^2=u^2+v^2
% uses DiagonalPowerSpectralDensityLaw2D: definition of radial PSD

    % x, y and frequency grids;
    [xgrid,ygrid] = meshgrid(0:deltaf:deltaf*(N-1),0:deltaf:deltaf*(N-1));
    fgrid = sqrt(xgrid.^2+ygrid.^2);

    % tabs of gaussian random numbers
    [tabx,taby] = gaussian(N,0,1/sqrt(2));

    lsd = DiagonalPSDLaw2D(fgrid);
    lsd(isinf(lsd)) = 0;
    lsd(isnan(lsd)) = 0;

    tab = complex(tabx,taby) .* sqrt(lsd);
end

function [y1,y2]=gaussian(N,moy,sigma)
%[y1,y2]=gaussian(N,moy,sigma)
% input:  N, mean, standard deviation
% output: 2 N*N tables of random numbers

    x1 = rand(N); % table of N*N random numbers
    x2 = rand(N); % an other one

    y1 = sqrt(-2*log(x1)).*cos(2*pi*x2)*sigma+moy;
    y2 = sqrt(-2*log(x1)).*sin(2*pi*x2)*sigma+moy;

end

%{
function spec=DiagonalPSDLaw2D(f)

global PSD_Freq
global map_PSD_2D
global law
% spec=DiagonalPSDLaw2D(f)
% linear spectral density dummy spectrum ressembling VSR1 NI and WI mirror
%    coating 2D linear spectral densities
% in m^2/m^{-2}
% single sideband
% change this file for other laws (corrective coating, etc.)

%law = 'vsr1'; % : 2D PSD observed on VSR1 mirrors
FrequencyScale = 80; % high pass filter:
%law = 'rich3'; % vsr1 + high pass filter below FrequencyScale
% law == 'test' % : white noise

switch law
    case 'rich7' % 
    spec=f;

    mysize= size(map_PSD_2D);
    fmin=zeros(mysize(1)-1,1);
    fmax=zeros(mysize(1)-1,1);
    fac=zeros(mysize(1)-1,1);
    pow=zeros(mysize(1)-1,1);
    for j=2:mysize(1)
        coeff=[map_PSD_2D(j-1,1) 1 ; map_PSD_2D(j,1) 1 ]\[map_PSD_2D(j-1,2) ; map_PSD_2D(j,2) ];
        pow(j-1)=coeff(1);
        fac(j-1)=10^(coeff(2));
        fmax(j-1)=10^(map_PSD_2D(j,1));
    end

    fi = find(f<=fmax(1));
    spec(fi) = fac(1)*(f(fi)).^(pow(1));
    for j=2:length(pow)-1
        fi = find(f<=fmax(j) & f>fmax(j-1));
        spec(fi) = fac(j)*(f(fi)).^(pow(j));
    end
    fi = find(f>fmax(length(pow)-1));
    spec(fi) = fac(length(pow))*(f(fi)).^(pow(length(pow)));

    spec = spec./(f*pi);

    fi = find(f==0);
    spec(fi) = 0;

    case 'rich6' % 
        spec=f;

        sig = [ ...
                1.097 -17.9451
                1.70639 -18.187
                2.14356 -20.7945
                2.673 -21.7581
                2.9169 -21.7117
                3.13881 -23.3883 ];
        mysize= size(sig);
        fmin=zeros(mysize(1)-1,1);
        fmax=zeros(mysize(1)-1,1);
        fac=zeros(mysize(1)-1,1);
        pow=zeros(mysize(1)-1,1);
        for j=2:mysize(1)
            coeff=[sig(j-1,1) 1 ; sig(j,1) 1 ]\[sig(j-1,2) ; sig(j,2) ];
            pow(j-1)=coeff(1);
            fac(j-1)=10^(coeff(2));
            fmax(j-1)=10^(sig(j,1));
        end

        fi = find(f<=fmax(1));
        spec(fi) = fac(1)*(f(fi)).^(pow(1));
        for j=2:length(pow)-1
            fi = find(f<=fmax(j) & f>fmax(j-1));
            spec(fi) = fac(j)*(f(fi)).^(pow(j));
        end
        fi = find(f>fmax(length(pow)-1));
        spec(fi) = fac(length(pow))*(f(fi)).^(pow(length(pow)));
       
        spec = spec./(f*pi);

        fi = find(f==0);
        spec(fi) = 0;
        
 
    case 'rich5' % 
        spec=f;

        fi = find(f>0);
        spec(fi) = 1e-22./(pi*f(fi));

        fi = find(f==0);
        spec(fi) = 0;

    case 'rich4' % 
        
        spec=f;
        fi = find(f>0);
        spec(fi) = 0;
 
        fi = find(f>50);
        spec(fi) = 1e-21./f(fi);

         fi = find(f>100);
        spec(fi) = 1e-22./f(fi);

        fi = find(f>400);
        spec(fi) = 0;
     
        fi = find(f==0);
        spec(fi) = 0;
        
    case 'rich3' % 
        spec=f;
       
        fi = find(f<PSD_Freq-10);
        spec(fi) = 0;

        fi = find(f>=PSD_Freq-10);
        spec(fi) = 1.6e-20./f(fi);

        fi = find(f>(PSD_Freq+10));
        spec(fi) = 0;

        fi = find(f==0);
        spec(fi) = 0;
        
   case 'rich2' % 
        spec=f;

        load('PSD.mat');
 
        mysize= size(sig);
        fmin=zeros(mysize(1)-1,1);
        fmax=zeros(mysize(1)-1,1);
        fac=zeros(mysize(1)-1,1);
        pow=zeros(mysize(1)-1,1);
        for j=2:mysize(1)
            coeff=[sig(j-1,1) 1 ; sig(j,1) 1 ]\[sig(j-1,2) ; sig(j,2) ];
            pow(j-1)=coeff(1);
            fac(j-1)=10^(coeff(2));
            fmax(j-1)=10^(sig(j,1));
        end

        fi = find(f<=fmax(1));
        spec(fi) = fac(1)*(f(fi)).^(pow(1));
        for j=2:length(pow)-1
            fi = find(f<=fmax(j) & f>fmax(j-1));
            spec(fi) = fac(j)*(f(fi)).^(pow(j));
        end
        fi = find(f>fmax(length(pow)-1));
        spec(fi) = fac(length(pow))*(f(fi)).^(pow(length(pow)));
       
        fi = find(f==0);
        spec(fi) = 0;
       
    case 'rich' % 
%         load('PSD.mat');
%         mys=size(f);
%         spec=zeros(mys);
%         for j=1:mys(2);
%             spec(:,j) = interp1(rho_axis,Res_PSD_1D,f(:,j));
%         end
        spec=f;

%         sig = [ ...
%                 1.631 -21.1 
%                 2.624 -24.9 
%                 2.939 -25.16 
%                 3.27  -28.47  ];
%             
%            1.851 -22.24      
%            2.634 -25.21     
%            2.94 -25.47      
%            3.185 -27.92       ];
        sig = [ ...
                1.148 -20.29
                 1.626 -20.56
                 1.844 -21.25
                 2.193 -23.25
                 2.62 -24.79
                 2.93 -25.15
                 3.16 -27.22 ];
        mysize= size(sig);
        fmin=zeros(mysize(1)-1,1);
        fmax=zeros(mysize(1)-1,1);
        fac=zeros(mysize(1)-1,1);
        pow=zeros(mysize(1)-1,1);
        for j=2:mysize(1)
            coeff=[sig(j-1,1) 1 ; sig(j,1) 1 ]\[sig(j-1,2) ; sig(j,2) ];
            pow(j-1)=coeff(1);
            fac(j-1)=10^(coeff(2));
            fmax(j-1)=10^(sig(j,1));
        end

        fi = find(f<=fmax(1));
        spec(fi) = fac(1)*(f(fi)).^(pow(1));
        for j=2:length(pow)-1
            fi = find(f<=fmax(j) & f>fmax(j-1));
            spec(fi) = fac(j)*(f(fi)).^(pow(j));
        end
        fi = find(f>fmax(length(pow)-1));
        spec(fi) = fac(length(pow))*(f(fi)).^(pow(length(pow)));
       
        fi = find(f==0);
        spec(fi) = 0;
 
%          fi = find(f<50);
%          spec(fi) = 0;
        % 0 is actually surface average
    case 'vsr1' % PSD for mirrors with polishing like the one used in VSR1

        spec=f;

        fi = find(f>800);
        spec(fi) = 1.8e-26*(f(fi)/800).^(-9);

        fi = find(f<=800 & f>160);
        spec(fi) = 1.8e-26*((f(fi)/800).^(-2.5));

        fi = find(f<=160 & f>140);
        spec(fi) = 1.006e-24*(f(fi)/160).^8;

        fi = find(f<=140 & f>0);
        spec(fi) = 6.003e-25*(f(fi)/140).^(-3.7);

        fi = find(f==0);
        spec(fi) = 0;
        % 0 is actually surface average

    case 'corrective coating'

        % all spatial frequencies below scale have PSD = PSD(scale)
        spec=f;
        fi = find(f>800);
        spec(fi) = 1.8e-26*(f(fi)/800).^(-9);
        fi = find(f<=800 & f>160);
        spec(fi) = 1.8e-26*((f(fi)/800).^(-2.5));
        fi = find(f<=160 & f>140);
        spec(fi) = 1.006e-24*(f(fi)/160).^8;
        fi = find(f<=140 & f>0);
        spec(fi) = 6.003e-25*(f(fi)/140).^(-3.7);

        if (FrequencyScale>800)
            lowfqdsp = 1.8e-26*(FrequencyScale/800).^(-9);
        elseif (FrequencyScale<=800 & FrequencyScale>160)
            lowfqdsp = 1.8e-26*((FrequencyScale/800).^(-2.5));
        elseif (FrequencyScale<=160 & FrequencyScale>140)
            lowfqdsp = 1.006e-24*(FrequencyScale/160).^8;
        elseif (FrequencyScale<=140 & FrequencyScale>0);
            lowfqdsp = 6.003e-25*(FrequencyScale/140).^(-3.7);
        end

        ind = find(f<FrequencyScale);
        ind = ind(2:end); % exclude f=0
        spec(ind) = lowfqdsp.*(FrequencyScale./f(ind));

        fi = find(f==0);
        spec(fi) = 0;


    case 'test'
        %for test purposes: white noise, rms deviation=1
        [n,m] = size(f);
        spec = ones(n,m)*2;
        fi = find(f==0);
        spec(fi) = 0;
end
end
%}