%
% fill data points with NaN value by averages of valid values around the NaN point.
% The filling is applied only for those NaN points surrounded by valid values.
%
% January 8, 2014 by Hiro Yamamoto
%
function datOut = fillNaN( datIn )

datOut = datIn;

for idx = 1 : size(datIn,2)
    % find region, ind1:ind2, where ind1 and ind2 are valid poits
    % and analyze NaN values in ind1+1:ind2-1
    ind1 = find( ~isnan( datIn(:,idx) ), 1 );
    if ~isempty( ind1 )
        % there is some valid data in this column
        ind1 = ind1 + 1;
        % find the last valid value
        ind2 = find( ~isnan( datIn(:,idx) ), 1, 'last' ) - 1;
        % we take care of NaNs between ind1 and ind2, outmost valid values
        indNaN = find( isnan( datIn(ind1:ind2, idx) ) );
        
        if ~isempty( indNaN )
            % there are some NaNs between ind1 and ind2
            % replace these NaNs by the average of values around = sum( val / distance )
            for id = 1:length(indNaN)
                idy = indNaN(id) + ind1 - 1; 
                % these points have good values along y direction, but may not be in x direction
                % so, first sum along x direction
                % find the valid data along -x direction
                offValid = find( ~isnan( datIn(idy, idx-1:-1:1) ), 1 );
                if ~isempty( offValid )
                    % there is a valid data
                    sumWeight = 1/offValid;
                    sumVal = datIn( idy, idx-offValid ) / offValid;
                    % fin dthe valid data along +x direction
                    offValid = find( ~isnan( datIn(idy, idx+1:end) ), 1 );
                    if ~isempty( offValid )
                        % there is valid data in this direction as well
                        sumWeight = sumWeight + 1/offValid;
                        sumVal = sumVal + datIn( idy, idx+offValid ) / offValid;
                        % include values along -y and +y direction in the average calculation
                        offValid = find( ~isnan( datIn(idy-1:-1:1, idx) ), 1 );
                        sumWeight = sumWeight + 1/offValid;
                        sumVal = sumVal + datIn( idy - offValid, idx ) / offValid;
                        offValid = find( ~isnan( datIn(idy+1:end, idx) ), 1 );
                        sumWeight = sumWeight + 1/offValid;
                        sumVal = sumVal + datIn( idy + offValid, idx ) / offValid;
                        
                        datOut( idy, idx ) = sumVal / sumWeight;
                    end
                end
            end
        end
    end
end
