function [array, wavelength, aspect, pixel] = readopd(filename)
%[array, wavelength, aspect, pixel] = readopd(filename)
%
%READOPD Reads WYKO OPD file into an array
%
%	[array,wavelength,aspect] = READOPD(filename)
%	returns array of phase data, wavelength and aspect ratio
%	stored in the opd file.

% 	Artur Olszak
%	WYKO Corporation, Tuesday, March 17, 1998
%

% open the file
fid=fopen (filename, 'r');
msg=sprintf ('can not open file: %s', filename);
if fid==-1 
	error (msg); 
end;

% **************read the block file****************
% first read the signature
tmp=fread(fid, 2, 'char');
%if tmp!='' error ('Not an Vision OPD file, aborting...'); end

% read directory 
% 
dirname =   fread(fid, 16, 'char');
dirname = deblank(setstr (dirname'));
dirtype =   fread(fid, 1,  'short');
dirlength = fread(fid, 1,  'long');
dirattr =   fread(fid, 1,  'ushort');

if strcmp(dirname,'Directory')==0 
	error ('Unable to read Directory entry, aborting...'); 
end;

%calculate number of directory entries
num_entries=dirlength/24;

counter=1;
mult_offset = 0;
offset = dirlength+2;

%read all directory entries
for i=2:num_entries
	dirname =   fread(fid, 16, 'char');
	dirname = deblank(setstr (dirname'));
	%if strcmp(dirname,'Wavelength')==1 wavelength;
	dirtype =   fread(fid, 1,  'short');
	dirlength = fread(fid, 1,  'long');
	dirattr =   fread(fid, 1,  'ushort');
	if length(dirname)>0
		if dirtype==7
			type='Float array 2D';
		elseif dirtype==3
			type='Array 3D';
		else
			type='Unknown';
		end
		inf=sprintf('Data: %-20s %-8u %20s %-5u', dirname, dirlength, type, dirattr);
%		disp(inf);
		counter = counter+1;

		if (strcmp(dirname,'OPD')==1)
			rawdata_offset = offset;
			rawdata_type = dirtype;
		end;
        if strcmp(dirname,'RAW_DATA')==1
            rawdata_offset = offset;
			rawdata_type = dirtype;
        end;
		if strcmp(dirname,'Mult')==1 
			mult_offset = offset;
		end;
		if strcmp(dirname,'Wavelength')==1 
			wavelength_offset = offset;
		end;
		if strcmp(dirname,'Aspect')==1 
			aspect_offset = offset;
		end;
        if strcmp(dirname,'Pixel_size') == 1
            Pixel_offset = offset;
        end
	end;
	
	offset = offset+dirlength;
end;

% disp(sprintf('total of %u nonzero entries', counter));

% read the data from data blocks

%if mult_offset!=0
%	fseek(fid, mult_offset, 'bof')
%	mult = fread(fid, 1, 'short');
%end;

% RAW_DATA
fseek(fid, rawdata_offset, 'bof');
xsize  = fread(fid, 1,  'short');
ysize  = fread(fid, 1,  'short');
elsize = fread(fid, 1,  'short');
if elsize==1
	type='char';
elseif elsize==2
	type='short';
elseif elsize==4
	type='float';
end;
array  = fread(fid, [ysize xsize], type);
array=array';

% read wavelength
fseek(fid, wavelength_offset, 'bof');
wavelength=1e-9*fread(fid, 1, 'float');

fseek(fid, aspect_offset, 'bof');
aspect=fread(fid, 1, 'float');

fseek(fid, Pixel_offset, 'bof');
pixel=fread(fid, 1, 'float') * 1e-3;

% [m n]=size(array);
% for i=1:m
% 	for j=1:n
% 		if array(i,j)>1e38
% 			array(i,j)=0;
% 		end
% 	end
% end


fclose(fid);
