% function map0 = removePower(map, x, y, aperture, powerOrAbr )
%  remove power and/or sphericaql aberration
%  aperture : used for calculating power and aberration by zernikeCoef
%  powerOrAbr : -1 : do nothing
%                0 : remove power
%                > 0 : remove power and spherical aberration
%                < 0 : remove power
%      r > abs(powerOrAbr) region is filled average value in abs(powerOrAbr) ~ abs(powerOrAbr)+1cm
%
% map0 = removePower(map, x, y, aperture, -1 ) : do nothing
% map0 = removePower(map, x, y, aperture,  0 ) : remove power
% map0 = removePower(map, x, y, aperture,  -0.1 ) : remove power and fill r>0.1 by the value average of 0.1 to 0.11
% map0 = removePower(map, x, y, aperture,   0.1 ) : remove power and spherical aberration and fill r>0.1 by the value average of 0.1 to 0.11

function map0 = removePower(map, x, y, aperture, powerOrAbr )

if ~exist( 'powerOrAbr', 'var' )
    powerOrAbr = 0;
end
if powerOrAbr == -1
    map0 = map;
    return;
end

if powerOrAbr ~= 0
    ind0 = find( x.^2 + y.^2 > powerOrAbr^2 & x.^2 + y.^2 < (abs(powerOrAbr)+0.01)^2 ); % thin band between adius and radius + 1cm
    ind1 = find( x.^2 + y.^2 > powerOrAbr^2 );  % outside of radius
else
    ind0 = 0; ind1 = 0;
end

if powerOrAbr > 0  % if powerOrAbr > 0, remove power and spherical aberration
    map0 = zernikeCoef( map, x(1,:), y(:,1), aperture, [2,0; 4,0] );
else  % if powerOrAbr <= 0, remove only power
    map0 = zernikeCoef( map, x(1,:), y(:,1), aperture, [2,0] );
end

% if powerOrAbr ~= 0, fill outside by the mean around the edge
if ind0 ~= 0
    map0( ind1 ) = mean( map0(ind0 ) );
end
