% function Etrace = traceProp( ifo, gaussCase, startFld, endFld )
%
% starting from startFld, go following reflection, until endFld
% if endFld is not specified, this ends at the reverse of the statFld,
% like starts from ITM-HR-o and ends at ITM-HR-i
%
% gaussCase : condition of the initial field
%  1 => full gauss, 
% -1 => truncated Gauss, (full gaussian is reflected by the first optic)
%  0 => calculated field
%
% March 23, 2017
%   First version by Hiro Yamamoto
%
function Etrace = traceProp( ifo, gaussCase, startFld, endFld )

Etrace = repmat( struct( 'ID', ifo.fields.findNamedField( startFld, -1 ), ...
    'E', ifo.ffts.zeroMat, 'x', zeros(1,size(ifo.ffts.zeroMat,2)) ), 1 );
id = 1;
done = false;
if ~exist( 'endFld', 'var' )
    endFld = ifo.optics.findReverseID(Etrace(1).ID);
else
    endFld = ifo.fields.findNamedField( endFld, -1 );
end
for id = 1 : length(ifo.fields.fieldList)
    if id == 1
        if gaussCase > 0
            [E, x] = ifo.fields.fieldAtFID( Etrace(1).ID );
        elseif gaussCase < 0
            fldID = ifo.optics.findReflPort(Etrace(1).ID, true);
            [E, x] = ifo.fields.fieldAtFID(fldID);
            E = ifo.fields.fieldList(fldID).conv2int(E);
            Etrace(id).E = ifo.optics.refl( fldID, E );
        else
            [E, x] = ifo.getField(Etrace(1).ID);
        end
        if gaussCase >= 0
            Etrace(1).E = ifo.fields.fieldList(Etrace(1).ID).conv2int(E);
        end
        Etrace(1).x = x;
    elseif mod(id,2) == 0 % propagation
        [sinkID, propID] = ifo.props.getPropTo( Etrace(id-1).ID );
        Etrace(id).ID = sinkID;
        Etrace(id).E = ifo.props.propField( propID, Etrace(id-1).E );
        Etrace(id).x = ifo.fields.getXlist(sinkID);
    else % reflection
        [optID, portIndex] = ifo.optics.findOptic( Etrace(id-1).ID );
        Etrace(id).ID = ifo.optics.optList(optID).outPorts(ifo.optics.optList(optID).reflMatch(abs(portIndex)));
        Etrace(id).E = ifo.optics.refl( Etrace(id-1).ID, Etrace(id-1).E );
        Etrace(id).x = ifo.fields.getXlist(Etrace(id).ID);
    end
    if id > 1 && Etrace(id).ID == endFld
        break;
    end
end

for id = 1 : length( Etrace )
    Etrace(id).E = ifo.fields.fieldList(Etrace(id).ID).conv2std(Etrace(id).E);
end


